@@ -19,7 +19,7 @@ use Image::ExifTool qw(:DataAccess);
 use Image::ExifTool::Canon;
 use Image::ExifTool::Exif;
 
-$VERSION = '1.55';
+$VERSION = '1.58';
 
 sub ProcessCanonCustom($$$);
 sub ProcessCanonCustom2($$$);
@@ -1290,9 +1290,19 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         },
     },
     0x0106 => [{
+        Name => 'AEBShotCount',
+        Condition => '$$self{Model} =~ /\b90D\b/',
+        Notes => 'EOS 90D', # (and who knows what others?)
+        PrintConv => {
+            2 => '2 shots',
+            3 => '3 shots',
+            5 => '5 shots',
+            7 => '7 shots',
+        },
+    },{
         Name => 'AEBShotCount',
         Condition => '$count == 1',
-        Notes => 'one value for some models...',
+        Notes => 'other models storing a single value',
         PrintConv => {
             0 => '3 shots',
             1 => '2 shots',
@@ -1302,7 +1312,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     },{
         Name => 'AEBShotCount',
         Count => 2,
-        Notes => 'two values for others',
+        Notes => 'models storing two values',
         PrintConv => {
             '3 0' => '3 shots',
             '2 1' => '2 shots',
@@ -1319,14 +1329,20 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     },
     0x0108 => {
         Name => 'SafetyShift',
-        Notes => 'value of 2 not used by 40D, 50D, 60D, 5DmkII and 7D',
+        Notes => 'value of 2 not used by some models', # eg. 40D, 50D, 60D, 5DmkII, 7D, 250D
         PrintConv => {
             0 => 'Disable',
             1 => 'Enable (Tv/Av)',
             2 => 'Enable (ISO speed)',
         },
     },
-    0x0109 => {
+    0x0109 => [{ # (1DXmkIII)
+        Name => 'UsableShootingModes',
+        Condition => '$count == 1',
+        Count => 1,
+        PrintConv => 'sprintf("Flags 0x%x",$val)',
+        PrintConvInv => '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
+    },{
         Name => 'UsableShootingModes',
         Count => 2,
         PrintConv => [
@@ -1337,10 +1353,16 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             undef,
             '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
         ],
-    },
-    0x010a => {
+    }],
+    0x010a => [{ # (1DXmkIII)
         Name => 'UsableMeteringModes',
-        Count => 2,
+        Condition => '$count == 1',
+        Count => 1,
+        PrintConv => 'sprintf("Flags 0x%x",$val)',
+        PrintConvInv => '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
+    },{
+        Name => 'UsableMeteringModes',
+        Count => 2, 
         PrintConv => [
             \%disableEnable,
             'sprintf("Flags 0x%x",$val)', # (evaluative,partial,spot,center-weighted average)
@@ -1349,7 +1371,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             undef,
             '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
         ],
-    },
+    }],
     0x010b => {
         Name => 'ExposureModeInManual',
         PrintConv => {
@@ -1678,11 +1700,13 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
     0x040a => { # new for 5DmkIII
         Name => 'ViewfinderWarnings',
         PrintConv => { BITMASK => { #(NC)
-            0 => 'Monochrome',              # (have seen for: 5DmkII, 6D)
-            1 => 'WB corrected',            # (have seen for: 5DmkII, 6D)
+            0 => 'Monochrome',              # (have seen for: 5DmkII, 6D, 250D, 90D)
+            1 => 'WB corrected',            # (have seen for: 5DmkII, 6D, 90D)
             2 => 'One-touch image quality', # (have seen for: 5DmkII; doesn't exist for 6D)
             3 => 'ISO expansion',           # (have seen for: 5DmkII)
             4 => 'Spot metering',           # (have seen for: 5DmkII, 6D)
+            6 => 'Noise reduction',         # (have seen for: 250D, 90D)
+            7 => 'HDR',                     # (have seen for: 90D)
         }},
     },
     0x040b => { # new for 5DmkIII
@@ -1920,6 +1944,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 1 => 'AF point button: Auto selection; Rear dial: Manual selection',
             },
         },
+        # (this is 2 values for 90D,M6mkII, seen: "1046 1046")
     ],
     0x0510 => [ # new for 40D
         {
@@ -1963,6 +1988,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 1 => 'Enable',
                 2 => 'Register',
                 3 => 'Select AF-modes',
+                # also seen: 87 (90D), 1142 (RP)
             },
             'sprintf("Flags 0x%x",$val)', # (70D=Manual 1pt,Manual zone,Auto 19pt)
         ],
@@ -2062,7 +2088,45 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             2 => 'Enable: Down with Set',
         },
     },
-    0x0610 => {
+    0x0610 => [{ # (1DXmkIII)
+        Name => 'ContinuousShootingSpeed',
+        Condition => '$count == 6',
+        Count => 6,
+        PrintConv => [
+            \%disableEnable,
+            '"Hi $val"',
+            '"Cont $val"',
+            '"Lo $val"',
+            '"Soft $val"',
+            '"Soft LS $val"',
+        ],
+        PrintConvInv => [
+            undef,
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+        ],
+    },{ # (1DXmkIII firmware 1.3)
+        Name => 'ContinuousShootingSpeed',
+        Condition => '$count == 5',
+        Count => 5,
+        PrintConv => [
+            '"Hi $val"',
+            '"Cont $val"',
+            '"Lo $val"',
+            '"Soft $val"',
+            '"Soft LS $val"',
+        ],
+        PrintConvInv => [
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+            '$val=~/(\d+)/ ? $1 : 0',
+        ],
+    },{ # others
         Name => 'ContinuousShootingSpeed',
         Count => 3,
         PrintConv => [
@@ -2075,7 +2139,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             '$val=~/(\d+)/ ? $1 : 0',
             '$val=~/(\d+)/ ? $1 : 0',
         ],
-    },
+    }],
     0x0611 => {
         Name => 'ContinuousShotLimit',
         Count => 2,
@@ -2088,7 +2152,13 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             '$val=~/(\d+)/ ? $1 : 0',
         ],
     },
-    0x0612 => { # (1DX)
+    0x0612 => [{ # (1DXmkIII)
+        Name => 'RestrictDriveModes',
+        Condition => '$count == 1',
+        Count => 1,
+        PrintConv => 'sprintf("Flags 0x%x",$val)',
+        PrintConvInv => '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
+    },{ # (1DX)
         Name => 'RestrictDriveModes',
         Count => 2,
         PrintConv => [
@@ -2099,7 +2169,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             undef,
             '$val=~/0x([\dA-F]+)/i ? hex($1) : undef',
         ],
-    },
+    }],
     #### 4a) Operation
     0x0701 => [
         {
@@ -2116,6 +2186,17 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 3 => 'AE/AF, No AE lock',
             },
         },
+        {
+            Name => 'Shutter-AELock',
+            Condition => '$count == 2',
+            Notes => '250D',
+            PrintConv => {
+                '0 0' => 'AF/AE lock',
+                '1 0' => 'AE lock/AF',
+                '2 0' => 'AF/AF lock, No AE lock',
+                '3 0' => 'AE/AF, No AE lock',
+            },
+        },
         {
             Name => 'AFAndMeteringButtons',
             Condition => '$$self{Model} =~ /\b60D\b/',
@@ -2233,6 +2314,13 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
                 5 => 'Disabled',
             },
         },
+        {
+            Name => 'SetButtonWhenShooting',
+            Notes => '250D',
+            Condition => '$count == 2',
+            # (not sure how to decode this.  seen: "37 0")
+            PrintConv => { },
+        },
         {
             Name => 'SetButtonWhenShooting',
             Notes => '1DmkIII and 1DmkIV',
@@ -2436,7 +2524,21 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             },
         },
     ],
-    0x080c => {
+    0x080c => [{ # (1DXmkIII)
+        Name => 'TimerLength',
+        Condition => '$count == 3',
+        Count => 3,
+        PrintConv => [
+            '"6 s: $val"',
+            '"16 s: $val"',
+            '"After release: $val"',
+        ],
+        PrintConvInv => [
+            '$val=~/(\d+)$/ ? $1 : 0',
+            '$val=~/(\d+)$/ ? $1 : 0',
+            '$val=~/(\d+)$/ ? $1 : 0',
+        ],
+    },{
         Name => 'TimerLength',
         Count => 4,
         PrintConv => [
@@ -2451,7 +2553,7 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
             '$val=~/(\d+)$/ ? $1 : 0',
             '$val=~/(\d+)$/ ? $1 : 0',
         ],
-    },
+    }],
     0x080d => {
         Name => 'ShortReleaseTimeLag',
         PrintConv => \%disableEnable,
@@ -2498,6 +2600,8 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         PrintConv => {
             0 => 'Cancel selected',
             1 => 'Erase selected',
+            2 => 'Erase RAW selected', # (1DXmkIII)
+            3 => 'Erase non-RAW selected', # (1DXmkIII)
         },
     },
     0x0814 => { # (5DS)
@@ -2508,6 +2612,10 @@ my %convPFn = ( PrintConv => \&ConvertPfn, PrintConvInv => \&ConvertPfnInv );
         Name => 'AddIPTCInformation',
         PrintConv => \%disableEnable,
     },
+    0x0816 => { # (90D,RP)
+        Name => 'AudioCompression',
+        PrintConv => \%enableDisable,
+    }
 );
 
 #------------------------------------------------------------------------------
@@ -2569,7 +2677,7 @@ sub ProcessCanonCustom2($$$)
         my $recPos = $pos;
         my $recEnd = $pos + $recLen - 8;
         if ($recEnd > $end) {
-            $et->Warn('Corrupted CanonCustom2 group');
+            $et->Warn("Corrupted CanonCustom2 group $recNum");
             return 0;
         }
         if ($verbose and not $write) {
@@ -2579,8 +2687,17 @@ sub ProcessCanonCustom2($$$)
         for ($i=0; $recPos + 8 < $recEnd; ++$i, $recPos+=4*$num) {
             $tag = Get32u($dataPt, $recPos);
             $num = Get32u($dataPt, $recPos + 4);
+            my $nextRec = $recPos + 8 + $num * 4;
+            last if $nextRec > $recEnd;
+            # patch for EOS-1DXmkIII firmware 1.0.0 bug:
+            if ($tag == 0x70c and $num == 0x66 and $nextRec + 8 < $recEnd) {
+                my $tmp = Get32u($dataPt, $nextRec + 4);
+                if ($tmp == 0x70f) {
+                    $et->Warn('Fixed incorrect CanonCustom tag 0x70c size', 1);
+                    ++$num; # (count should be one greater)
+                }
+            }
             $recPos += 8;
-            last if $recPos + $num * 4 > $recEnd;
             my $val = ReadValue($dataPt, $recPos, 'int32s', $num, $num * 4);
             if ($write) {
                 # write new value
@@ -2612,6 +2729,9 @@ sub ProcessCanonCustom2($$$)
                 }
             }
         }
+        if ($i != $recCount) {
+            $et->Warn('Missing '.($recCount-$i)." entries in CanonCustom2 group $recNum directory", 1);
+        }
         $pos = $recEnd;
     }
     if ($pos != $end) {
@@ -2753,7 +2873,7 @@ Image::ExifTool to read this information.
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
