@@ -5,12 +5,12 @@
 #
 # Revisions:    2013/03/28 - P. Harvey Created
 #
-# References:   1) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,4898.0.html
+# References:   1) https://exiftool.org/forum/index.php/topic,4898.0.html
 #               2) http://www.nuage.ch/site/flir-i7-some-analysis/
 #               3) http://www.workswell.cz/manuals/flir/hardware/A3xx_and_A6xx_models/Streaming_format_ThermoVision.pdf
 #               4) http://support.flir.com/DocDownload/Assets/62/English/1557488%24A.pdf
 #               5) http://code.google.com/p/dvelib/source/browse/trunk/flirPublicFormat/fpfConverter/Fpfimg.h?spec=svn3&r=3
-#               6) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5538.0.html
+#               6) https://exiftool.org/forum/index.php/topic,5538.0.html
 #               JD) Jens Duttke private communication
 #
 # Glossary:     FLIR = Forward Looking Infra Red
@@ -24,7 +24,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::GPS;
 
-$VERSION = '1.18';
+$VERSION = '1.20';
 
 sub ProcessFLIR($$;$);
 sub ProcessFLIRText($$$);
@@ -64,7 +64,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
         Name => 'ImageTemperatureMax',
         %temperatureInfo,
         Notes => q{
-            these temperatures may be in Celcius, Kelvin or Fahrenheit, but there is no
+            these temperatures may be in Celsius, Kelvin or Fahrenheit, but there is no
             way to tell which
         },
     },
@@ -99,7 +99,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     NOTES => q{
         Information extracted from FLIR FFF images and the APP1 FLIR segment of JPEG
         images.  These tags may also be extracted from the first frame of an FLIR
-        SEQ file.
+        SEQ file, or all frames if the ExtractEmbedded option is used.
     },
     "_header" => {
         Name => 'FFFHeader',
@@ -378,7 +378,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     NOTES => q{
         FLIR camera information.  The Planck tags are variables used in the
         temperature calculation.  See
-        L<http://u88.n24.queensu.ca/exiftool/forum/index.php?topic=4898.msg23972#msg23972>
+        L<https://exiftool.org/forum/index.php?topic=4898.msg23972#msg23972>
         for details.
     },
     0x00 => {
@@ -691,7 +691,15 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
             T => 'True North',
         },
     },
-  # 0x48 - int32u: seen 0,77
+    0x48 => { #PH (NC)
+        Name => 'GPSImgDirectionRef',
+        Format => 'string[2]',
+        RawConv => 'length($val) ? $val : undef',
+        PrintConv => {
+            M => 'Magnetic North',
+            T => 'True North',
+        },
+    },
     0x4c => {
         Name => 'GPSSpeed',
         %float2f,
@@ -702,7 +710,11 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
         %float2f,
         RawConv => '$val < 0 ? undef : $val',
     },
-  # 0x54 - float: seen 0,-1
+    0x54 => {
+        Name => 'GPSImgDirection',
+        %float2f,
+        RawConv => '$val < 0 ? undef : $val',
+    },
     0x58 => {
         Name => 'GPSMapDatum',
         Format => 'string[16]',
@@ -715,7 +727,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
 );
 
 # humidity meter information
-# (ref http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5325.0.html)
+# (ref https://exiftool.org/forum/index.php/topic,5325.0.html)
 # The %Image::ExifTool::UserDefined hash defines new tags to be added to existing tables.
 %Image::ExifTool::FLIR::MeterLink = (
     GROUPS => { 0 => 'APP1', 2 => 'Image' },
@@ -754,7 +766,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     96 => {
         Name => 'Reading1Value',
         Format => 'double',
-        # convert Kelvin -> Celcius and kg/kg -> g/kg
+        # convert Kelvin -> Celsius and kg/kg -> g/kg
         ValueConv => q{
             return $val - 273.15 if $$self{Reading1Units} == 0x0d and $$self{Reading1Description} != 11;
             return $val *= 1000 if $$self{Reading1Units} == 0x24;
@@ -794,7 +806,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     196 => {
         Name => 'Reading2Value',
         Format => 'double',
-        # convert Kelvin -> Celcius and kg/kg -> g/kg
+        # convert Kelvin -> Celsius and kg/kg -> g/kg
         ValueConv => q{
             return $val - 273.15 if $$self{Reading2Units} == 0x0d and $$self{Reading2Description} != 11;
             return $val *= 1000 if $$self{Reading2Units} == 0x24;
@@ -833,7 +845,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     296 => {
         Name => 'Reading3Value',
         Format => 'double',
-        # convert Kelvin -> Celcius and kg/kg -> g/kg
+        # convert Kelvin -> Celsius and kg/kg -> g/kg
         ValueConv => q{
             return $val - 273.15 if $$self{Reading3Units} == 0x0d and $$self{Reading3Description} != 11;
             return $val *= 1000 if $$self{Reading3Units} == 0x24;
@@ -873,7 +885,7 @@ my %float8g = ( Format => 'float', PrintConv => 'sprintf("%.8g",$val)' );
     396 => {
         Name => 'Reading4Value',
         Format => 'double',
-        # convert Kelvin -> Celcius and kg/kg -> g/kg
+        # convert Kelvin -> Celsius and kg/kg -> g/kg
         ValueConv => q{
             return $val - 273.15 if $$self{Reading4Units} == 0x0d and $$self{Reading4Description} != 11;
             return $val *= 1000 if $$self{Reading4Units} == 0x24;
@@ -1400,7 +1412,6 @@ sub ProcessMeasInfo($$$)
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dataPos = $$dirInfo{DataPos};
     my $dirEnd = $dirStart + $$dirInfo{DirLen};
-    my $verbose = $et->Options('Verbose');
 
     my $pos = $dirStart + 12;
     return 0 if $pos > $dirEnd;
@@ -1458,6 +1469,7 @@ sub ProcessFLIR($$;$)
     my $raf = $$dirInfo{RAF} || new File::RandomAccess($$dirInfo{DataPt});
     my $verbose = $et->Options('Verbose');
     my $out = $et->Options('TextOut');
+    my $base = $raf->Tell();
     my ($i, $hdr, $buff, $rec);
 
     # read and verify FFF header
@@ -1486,15 +1498,18 @@ sub ProcessFLIR($$;$)
         my $ver = Get32u(\$hdr, 0x14);
         last if $ver >= 100 and $ver < 200; # (have seen 100 and 101 - PH)
         ToggleByteOrder();
-        $i and $et->Warn("Unsupported FLIR $type version"), return 1;
+        next unless $i;
+        return 0 if $$et{DOC_NUM};
+        $et->Warn("Unsupported FLIR $type version");
+        return 1;
     }
 
     # read the FLIR record directory
     my $pos = Get32u(\$hdr, 0x18);
     my $num = Get32u(\$hdr, 0x1c);
-    unless ($raf->Seek($pos) and $raf->Read($buff, $num * 0x20) == $num * 0x20) {
+    unless ($raf->Seek($base+$pos) and $raf->Read($buff, $num * 0x20) == $num * 0x20) {
         $et->Warn('Truncated FLIR FFF directory');
-        return 1;
+        return $$et{DOC_NUM} ? 0 : 1;
     }
 
     unless ($tagTablePtr) {
@@ -1505,6 +1520,7 @@ sub ProcessFLIR($$;$)
     # process the header data
     $et->HandleTag($tagTablePtr, '_header', $hdr);
 
+    my $success = 1;
     my $oldIndent = $$et{INDENT};
     $$et{INDENT} .= '| ';
     $et->VerboseDir($type, $num);
@@ -1534,12 +1550,22 @@ sub ProcessFLIR($$;$)
         $verbose and printf $out "%s%d) FLIR Record 0x%.2x, offset 0x%.4x, length 0x%.4x\n",
                                  $$et{INDENT}, $i, $recType, $recPos, $recLen;
 
-        unless ($raf->Seek($recPos) and $raf->Read($rec, $recLen) == $recLen) {
-            $et->Warn('Invalid FLIR record');
+        # skip RawData records for embedded documents
+        if ($recType == 1 and $$et{DOC_NUM}) {
+            $raf->Seek($base+$recPos+$recLen) or $success = 0, last;
+            next;
+        }
+        unless ($raf->Seek($base+$recPos) and $raf->Read($rec, $recLen) == $recLen) {
+            if ($$et{DOC_NUM}) {
+                $success = 0;   # abort processing more documents
+            } else {
+                $et->Warn('Invalid FLIR record');
+            }
             last;
         }
         if ($$tagTablePtr{$recType}) {
             $et->HandleTag($tagTablePtr, $recType, undef,
+                Base    => $base,
                 DataPt  => \$rec,
                 DataPos => $recPos,
                 Start   => 0,
@@ -1551,7 +1577,17 @@ sub ProcessFLIR($$;$)
     }
     delete $$et{SET_GROUP0};
     $$et{INDENT} = $oldIndent;
-    return 1;
+
+    # extract information from subsequent frames in SEQ file if ExtractEmbedded is used
+    if ($$dirInfo{RAF} and $et->Options('ExtractEmbedded') and not $$et{DOC_NUM}) {
+        for (;;) {
+            $$et{DOC_NUM} = $$et{DOC_COUNT} + 1;
+            last unless ProcessFLIR($et, $dirInfo, $tagTablePtr);
+            # (DOC_COUNT will be incremented automatically if we extracted any tags)
+        }
+        delete $$et{DOC_NUM};
+    }
+    return $success;
 }
 
 #------------------------------------------------------------------------------
@@ -1595,7 +1631,7 @@ Systems Inc. thermal image files (FFF, FPF and JPEG format).
 
 =head1 AUTHOR
 
-Copyright 2003-2019, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -1604,7 +1640,7 @@ under the same terms as Perl itself.
 
 =over 4
 
-=item L<http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,4898.0.html>
+=item L<https://exiftool.org/forum/index.php/topic,4898.0.html>
 
 =item L<http://www.nuage.ch/site/flir-i7-some-analysis/>
 
@@ -1614,7 +1650,7 @@ under the same terms as Perl itself.
 
 =item L<http://code.google.com/p/dvelib/source/browse/trunk/flirPublicFormat/fpfConverter/Fpfimg.h?spec=svn3&r=3>
 
-=item L<http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,5538.0.html>
+=item L<https://exiftool.org/forum/index.php/topic,5538.0.html>
 
 =back
 
