@@ -988,6 +988,12 @@ sub ProcessPhotoshop($$$)
         $size += 1 if $size & 0x01; # size is padded to an even # bytes
         $pos += $size;
     }
+    # warn about incorrect IPTCDigest
+    if ($$et{VALUE}{IPTCDigest} and $$et{VALUE}{CurrentIPTCDigest} and
+        $$et{VALUE}{IPTCDigest} ne $$et{VALUE}{CurrentIPTCDigest})
+    {
+        $et->WarnOnce('IPTCDigest is not current. XMP may be out of sync');
+    }
     delete $$et{LOW_PRIORITY_DIR}{'*'};
     return $success;
 }
@@ -1137,7 +1143,7 @@ be preserved when copying Photoshop information via user-defined tags.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
