@@ -454,7 +454,7 @@ sub WriteCanonRaw($$$)
                 # pad with original garbage in case it contained something useful
                 $value .= substr($buff, $pt+2+8-$len, 8-$len);
             } elsif ($len > 8) {   # this shouldn't happen
-                warn "Value too long! -- trucated\n";
+                warn "Value too long! -- truncated\n";
                 $value = substr($value, 0, 8);
             }
             # create new directory entry
@@ -623,7 +623,7 @@ JPEG files, and would lead to far fewer problems with corrupted metadata.
 
 =head1 AUTHOR
 
-Copyright 2003-2020, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2022, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
